#include "FileList.h"

namespace Torch {
   
/* compare is used by qsort as the sorting mechanism */
static int file_list_compare(const void *e1, const void *e2)
{
	return strcmp (*(char **) e1, *(char **) e2);
}

FileList::FileList(char *path_, int file_max)
{
	struct dirent * sdp;
	DIR *dirp;
	int count, i;

	path = NULL;
	filelist = NULL;

	if(path_[0] == '.')
	{
		warning("%s is not an absolute directory", path_);
	 	
		NbFiles = -1;
	}

	if(NbFiles != -1)
	{
		//
		// Get current directory
		//
		getwd(pwdStart);	
	
		path = (char *)allocator->alloc(strlen(path_)+2);
		strcpy(path, path_);
	
	
		stat(path, &st);
		if(S_ISREG(st.st_mode)) bOneFile = true;
		else bOneFile = false;

		if(bOneFile == false)
		{
			if(path[strlen(path) - 1] != '/') strcat(path, "/");
		
			//
			// make sure path is a directory
			//
			if(!S_ISDIR (st.st_mode))
			{
				warning("%s is not a directory", path);
		
				NbFiles = -1;

				goto file_list_stop;
			
			}

			//
			// make path the current working directory and open it
			//
			if(chdir(path) == -1 || (dirp = opendir(path)) == NULL)
	  		{
				perror(path);
		 
				NbFiles = -1;

				goto file_list_stop;
	  		}

			//
			// count number of files in path
			//
			for (count = 0; readdir(dirp) != NULL; count++);
			rewinddir (dirp);	

			print("%d files in %s ...\n", count, path);

			//
			// allocate memory to hold filenames in filelist
			//
			if ((filelist = (char **) allocator->alloc((count + 1) * sizeof (char *))) == NULL)
	  		{
				warning("Impossible to allocate directory entries");
		 
				NbFiles = -1;

				goto file_list_stop;
	  		}

			//
			// allocate memory for each filename
			//
			for (i = 0; (sdp = readdir (dirp)) != NULL; i++)
	  		{
				if ((filelist[i] = (char *) allocator->alloc((strlen (sdp->d_name) + 1) * sizeof (char))) == NULL)
				{
					//
					// make the best of any error here
					//
					warning("Impossible to allocate another filename");		 
					count = i;

					break;
				}

		 		//
		 		// copy filename
		 		//
		 		strcpy (filelist[i], sdp->d_name);
	  		}
			filelist[i] = NULL;

file_list_stop:
	
			if(NbFiles != -1)
			{
				//
				// sort filenames after '.' and '..'
				//
				qsort(&filelist[2], count - 2, sizeof (char *), file_list_compare);

				//
				// reset starting directory to write output files in the good place
				//
				chdir(pwdStart);

				NbFiles = count;	
				if(NbFiles <= 2) // 2 because of . and .. which are counted as files
	  			{
					warning("No files to process");

					NbFiles = 0;
	  			}
			}
		}
		else NbFiles = 1;

		if(NbFiles != -1)
		{
			if(bOneFile == false) 
			{
				if((file_max == -1) || (file_max > (NbFiles - 2))) 
					print("Indexing %d files or directories in %s ...\n", NbFiles - 2, path);	
				else 
				{
					print("Indexing %d files or directories over %d in %s ...\n", file_max, NbFiles - 2, path);	
	
					NbFiles = file_max + 2;
				}
			}
			else print("Indexing file %s ...\n", path);	
		}
	}
}

FileList::~FileList()
{
/*
	if(path != NULL) free(path);

	if(bOneFile == false)
	{
	 	if(filelist != NULL)
		{
			for(int i = 0 ; i < NbFiles ; i++) free((char *) filelist[i]);
			free((char **) filelist);
		}
	}
*/
}

bool FileList::loop()
{
	bool bKernel, bIsRegular;
	char *extension;
	char filename[FILE_LIST_PATH_MAX];
	char pathfilename[FILE_LIST_PATH_MAX];
	
	if(NbFiles <= 0)
	{
		print("No files to process\n");

		return false;
	}
	
	print("Listing %s\n", path);

	for(int i = 0 ; i < NbFiles ; i++)
	{
		if(bOneFile == false)
		{
			strcpy(filename, filelist[i]);
			
			strcpy(pathfilename, path);
			strcat(pathfilename, filename);

			if(stat(pathfilename, &st) == -1)
			{
				perror(filename);
			
				continue;
				
				bIsRegular = false;
			}
			else
			{
				//
				// The file is a regular file
				//
				if(S_ISREG (st.st_mode)) bIsRegular = true;
				else bIsRegular = false;
			}
		}
		else 
		{
		   	strcpy(pathfilename, path);
                
			if(stat(pathfilename, &st) == -1)
			{
				perror(filename);
			
				continue;
				
				bIsRegular = false;
			}
			else
			{
				//
				// The file is a regular file
				//
				if(S_ISREG (st.st_mode)) bIsRegular = true;
				else bIsRegular = false;
			}

			if(bIsRegular == true)
			{
				extension = (char *) rindex(path, '/');
		   
		 		if(extension != NULL)
                  		{                      
                        		extension++;

					strcpy(filename, extension);
					
					*extension = '\0';
                  		}
			}
		}
		
		if(bIsRegular == true)
		{
			bKernel = kernel(filename);
			if(bKernel == false) return false;
		}
	}
	
	return true;
}

}

