#ifndef IMAGE_DISK_X_FILE_INC
#define IMAGE_DISK_X_FILE_INC

#include "DiskXFile.h"

namespace Torch {

/** This class is designed to handle an image file on the disk
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class ImageDiskXFile : public DiskXFile
{
public:
   	/**@name data field */
        //@{
	/// loaded image
	unsigned char *pixmap;
	//@}
	
   	/**@name image size fields */
        //@{
   	/// width of the image 
	int width;

	/// height of the image
	int height;

	/// number of color components
	int n_planes;
	//@}

   	/**@name other fields */
        //@{
	/// filename of the image
	char *filename;
	
	/// coding of the colorspace
	char *coding;

	///
	bool verbose;
	//@}
	
	//-----

   	/**@name constructors and destructor */
        //@{
	/** Makes a ImageDiskXFile from an filename.
	
	    @param file_name is the filename of the image
	    @param open_flags is the opening mode of the file
	*/
	ImageDiskXFile(const char *file_name, const char *open_flags);

	/// Empty constructor
	ImageDiskXFile();

	/// Destructor
    	virtual ~ImageDiskXFile();
	//@}
	
   	/**@name read/write image header and image pixmap */
        //@{
	/** read the header from the file.
	
	    @param width_ is the width of the image read in the header
	    @param height_ is the height of the image read in the header
	*/
	virtual bool readHeader(int *width_, int *height_) = 0;

	/// read the #pixmap# from the file
	virtual int readPixmap() = 0;

	/** write the header from the file.
	
	    @param width_ is the width of the image written in the header
	    @param height_ is the height of the image written in the header
	*/
	virtual bool writeHeader(int width_, int height_) = 0;

	/// write the #pixmap_# into the file
	virtual int writePixmap(unsigned char *pixmap_) = 0;
	//@}
};

}

#endif
