#include "ipHisto.h"

namespace Torch {

ipHisto::ipHisto(int width_, int height_, const char *coding_) : ipCore(coding_)
{
	width = width_;
        height = height_;

	addBOption("normalise", &normalise, false, "normalise");

	seq_out = new(allocator) Sequence(n_planes, 256);
	
	for(int i=0; i<seq_out->n_frames; i++)
                for(int j=0; j<seq_out->frame_size; j++)
                        seq_out->frames[i][j] = 0.0;
}

void ipHisto::process(Sequence *seq_in)
{
	int input_size = width * height;
	int output_size = 256;
	
	switch(coding_id)
	{
	case 0:
      		for(int i = 0; i < input_size; i++)
        	{
                	int pixel = (int) seq_in->frames[0][i];

	                if(pixel > 255) pixel = 255;
        	        if(pixel < 0) pixel = 0;

	                seq_out->frames[0][pixel] += 1.0;
        	}

		if(normalise)
		{
			for(int i = 0; i < output_size ; i++)
			{
			   	int h = (int) seq_out->frames[0][i];

				if(h != 0) seq_out->frames[0][i] /= input_size;
			}
		}
		break;
	case 1:
	case 2:
      		for(int i = 0; i < input_size; i++)
        	{
	   		// R/Y value
	                int pixel = (int) seq_in->frames[0][3*i];

        	        if(pixel > 255) pixel = 255;
                	if(pixel < 0) pixel = 0;

	                seq_out->frames[0][pixel] += 1.0;

			// G/U value
                	pixel = (int) seq_in->frames[3*i+1];

	                if(pixel > 255) pixel = 255;
        	        if(pixel < 0) pixel = 0;

	                seq_out->frames[1][pixel] += 1.0;

			// G/V value
                	pixel = (int) seq_in->frames[3*i+2];

	                if(pixel > 255) pixel = 255;
        	        if(pixel < 0) pixel = 0;

	                seq_out->frames[2][pixel] += 1.0;
        	}

		if(normalise)
		{
			for(int i = 0; i < output_size; i++)
			{
			   	// histo R/Y
			   	int h = (int) seq_out->frames[0][i];
				if(h != 0) seq_out->frames[0][i] /= input_size;

			   	// histo G/U
			   	h = (int) seq_out->frames[1][i];
				if(h != 0) seq_out->frames[1][i] /= input_size;

			   	// histo B/V
			   	h = (int) seq_out->frames[2][i];
				if(h != 0) seq_out->frames[2][i] /= input_size;
			}
		}
		break;
	case 3:
		message("ipHisto::process() not implemented for float format");
		break;
	}
}

ipHisto::~ipHisto()
{
}

}

