#include "ipHistoEqual.h"

namespace Torch {

ipHistoEqual::ipHistoEqual(int width_, int height_, const char *coding_) : ipCore(coding_)
{
	width = width_;
        height = height_;
	
	histo_size = n_planes*256;		
	histogram = (real *)allocator->alloc(histo_size*sizeof(real));
	cumulhistogram = (real *)allocator->alloc(histo_size*sizeof(real));

	seq_out = new(allocator) Sequence(1, width*height*n_planes);
}

void ipHistoEqual::process(Sequence *seq_in)
{
	int input_size = width * height;
	
   	// init histograms
   	for(int i = 0; i < histo_size; i++)
	{
		histogram[i] = 0.0;
		cumulhistogram[i] = 0.0;
	}

	real *histo0_ = histogram;
	real *histo1_ = histogram + 256;
	real *histo2_ = histogram + 512;
	real *cumulhisto0_ = cumulhistogram;
	real *cumulhisto1_ = cumulhistogram + 256;
	real *cumulhisto2_ = cumulhistogram + 512;

	real sum;
	real sum0;
	real sum1;
	real sum2;

	switch(coding_id)
	{
	case 0:
		// computes histograms	
      		for(int i = 0; i < input_size; i++)
	      	{
        		int k = (int) seq_in->frames[0][i];

			if(k<0 || k>255) warning("ipHistoEqual::process() out-of-range value.");
			
			histogram[k] += 1.0 / input_size;
      		}

		// computes cumulative histogram
		sum = 0.0;
		for(int i = 0; i < 256; i++)
		{
			sum += histogram[i];
			cumulhistogram[i] = sum;
		}

		// normalize the histogram
		for(int i = 0; i < 256; i++)
			cumulhistogram[i] = 255.0 * (cumulhistogram[i] - cumulhistogram[0]) / cumulhistogram[255];
	      
		// 
		for(int i = 0; i < input_size; i++)
      		{
	        	int k = (int) seq_in->frames[0][i];
			seq_out->frames[0][i] = cumulhistogram[k];
	      	}
		break;
	case 1:
		// computes histograms	
      		for(int i = 0; i < input_size; i++)
	      	{
		   	int base = 3*i;
			
        		int k = (int) seq_in->frames[0][base];
			histo0_[k] += 1.0 / input_size;

        		k = (int) seq_in->frames[0][base+1];
			histo1_[k] += 1.0 / input_size;

        		k = (int) seq_in->frames[0][base+2];
			histo2_[k] += 1.0 / input_size;
      		}

		// computes cumulative histogram
		sum0 = 0.0;
		sum1 = 0.0;
		sum2 = 0.0;

		for(int i = 0; i < 256; i++)
		{
			sum0 += histo0_[i];
			sum1 += histo1_[i];
			sum2 += histo2_[i];

			cumulhisto0_[i] = sum0;
			cumulhisto1_[i] = sum1;
			cumulhisto2_[i] = sum2;
		}

		// normalize the histogram
		for(int i = 0; i < 256; i++)
		{
			cumulhisto0_[i] = 255.0 * (cumulhisto0_[i] - cumulhisto0_[0]) / cumulhisto0_[255];
			cumulhisto1_[i] = 255.0 * (cumulhisto1_[i] - cumulhisto1_[0]) / cumulhisto1_[255];
			cumulhisto2_[i] = 255.0 * (cumulhisto2_[i] - cumulhisto2_[0]) / cumulhisto2_[255];
		}
	
		// 
		for(int i = 0; i < input_size; i++)
      		{
		   	int base = 3*i;
			
	        	int k = (int) seq_in->frames[0][base];
			seq_out->frames[0][base] = cumulhisto0_[k];

	        	k = (int) seq_in->frames[0][base+1];
			seq_out->frames[0][base+1] = cumulhisto1_[k];

	        	k = (int) seq_in->frames[0][base+2];
			seq_out->frames[0][base+2] = cumulhisto2_[k];
	      	}
		break;
	case 2:
		message("ipHistoEqual::process() not implemented for YUV format");
		break;
	case 3:
		// computes histograms	
      		for(int i = 0; i < input_size; i++)
	      	{
        		int k = (int) (255.0 * seq_in->frames[0][i]);
			histogram[k] += 1.0 / input_size;
      		}

		// computes cumulative histogram
		sum = 0.0;
		for(int i = 0; i < 256; i++)
		{
			sum += histogram[i];
			cumulhistogram[i] = sum;
		}

		// normalize the histogram
		for(int i = 0; i < 256; i++)
			cumulhistogram[i] = (cumulhistogram[i] - cumulhistogram[0]) / cumulhistogram[255];
	      
		// 
		for(int i = 0; i < input_size; i++)
      		{
	        	int k = (int) (255.0 * seq_in->frames[0][i]);
			seq_out->frames[0][i] = cumulhistogram[k];
	      	}
		break;
	}
}

ipHistoEqual::~ipHistoEqual()
{
}

}

