#include "matrix_addons.h"

namespace Torch {

/** Vector-vector transposed multiplication.
    -- #X.Y^T# is stored in M_ */
void mxVecMulTrVec(Vec *X_, Vec *T_Y_, Mat *M_)
{
   	if(X_->n != T_Y_->n) error("Vector of different size in mxVecMulTrVec().");
	
	for(int i = 0 ; i < X_->n ; i++)
       		for(int j = 0 ; j < X_->n ; j++)
            		M_->ptr[i][j] = X_->ptr[i] * T_Y_->ptr[j];   
}

void printV(int n, float *vec)
{
	print("[ ");

	for(int n_ = 0 ; n_ < n ; n_++)
		print("%g ", vec[n_]);

	print("]\n");
}
			
void printV(int n, double *vec)
{
	print("[ ");

	for(int n_ = 0 ; n_ < n ; n_++)
		print("%g ", vec[n_]);

	print("]\n");
}
			
void printV(Vec *vec)
{
	print("[ ");

	for(int n_ = 0 ; n_ < vec->n ; n_++)
		print("%g ", vec->ptr[n_]);

	print("]\n");
}
			
void printM(Mat *mat)
{
	print("[ ");
	for(int m_ = 0 ; m_ < mat->m ; m_++)
	{
		print("( ");

		for(int n_ = 0 ; n_ < mat->n ; n_++)
			print("%g ", mat->ptr[m_][n_]);

		print(")\n");
	}
	print("]\n");
}

void mxCopy(Mat *A, Mat *B, int h_target, int w_target, int h_begin, int w_begin, int h_end, int w_end)
{
	int i, j, h, w, h_done, w_done;

	if ((h_target >= 0)&&(h_target < A->m)&&(w_target >= 0)&&(w_target < A->n)) 
	{
		if ((h_begin >= 0)&&(h_begin < B->m)&&(w_begin >= 0)&&(w_begin < B->n)) 
		{
			h = h_end-h_begin+1;
			w = w_end-w_begin+1;
			if ((h >= 1)&&(w >= 1)) 
			{
				h_done = h_target+h-1;
				w_done = w_target+w-1;
				if ((h_done < A->m)&&(w_done < A->n)) 
				{
					for (i=0;i<h;i++) 
					{
						for (j=0;j<w;j++) 
							A->ptr[i+h_target][j+w_target] = B->ptr[i+h_begin][j+w_begin];
					}
				}
			}
		}
	}
	else {
		printf("matrix dimension error!\n");
		exit(1);
	}
}

void mxMul__(double * dp1, double * dp2, double * out, int len)
{
  for (int i = 0; i < len; i++)
    out[i] = dp1[i] * dp2[i];
}

void mxMul__(float * dp1, float * dp2, float * out, int len)
{
  for (int i = 0; i < len; i++)
    out[i] = dp1[i] * dp2[i];
}

void mxMulMat(Mat * mat1, Mat * mat2, Mat * out)
{
	int m, n;

	m = mat1->m;
	n = mat1->n;
	for (int i = 0; i < m; i++)
		mxMul__(mat1->ptr[i], mat2->ptr[i], out->ptr[i], n);
}

void mxScale(Mat *mat, real scale_)
{
	real max_ = -INF;
	real min_ = INF;
	
	for(int i = 0 ; i < mat->m ; i++)
		for(int j = 0 ; j < mat->n ; j++)
		{
			if(mat->ptr[i][j] > max_) max_ = mat->ptr[i][j];
			if(mat->ptr[i][j] < min_) min_ = mat->ptr[i][j];
		}
	
	for(int i = 0 ; i < mat->m ; i++)
		for(int j = 0 ; j < mat->n ; j++)
			mat->ptr[i][j] = (real) scale_ * (mat->ptr[i][j] - min_) / (max_ - min_);
}

void vecScale(Vec *vec, real scale_)
{
  real max_ = -INF;
  real min_ = INF;

  for (int i = 0; i < vec->n; i++)
    {
      if (vec->ptr[i] > max_) max_ = vec->ptr[i];
      if (vec->ptr[i] < min_) min_ = vec->ptr[i];
    }
  
  for (int i = 0; i < vec->n; i++)
    vec->ptr[i] = (real) scale_ * (vec->ptr[i] - min_) / (max_ - min_);
  
}

}
