const char *help = "\
progname: xlist2pgm.cc\n\
code2html: This program reads a list of image and saves them as a pgm.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageGray.h"
#include "xtprobeImageDiskXFile.h"

#include "FileListCmdOption.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	bool verbose;
	bool savepgm;

	FileListCmdOption imagelist("imagelist", "the list of image filename or one image filename");
	imagelist.isArgument(true);

	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
	cmd.addCmdOption(&imagelist);
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");
  	cmd.addBCmdOption("-savepgm", &savepgm, false, "save as pgm");

	cmd.read(argc, argv);

  	ImageDiskXFile *image_file = NULL;
	Image *grayimage = NULL;

	grayimage = new ImageGray();
	grayimage->setBOption("verbose", verbose);

	for(int i = 0 ; i < imagelist.n_files ; i++)
	{
		image_file = new xtprobeImageDiskXFile(imagelist.file_names[i], "r");
		grayimage->loadImageXFile(image_file);
		delete image_file;

		if(verbose)
		{
			print("Image info (%s):\n", imagelist.file_names[i]);
			print("   width = %d\n", grayimage->width);
			print("   height = %d\n", grayimage->height);
			print("   format = %s (%d)\n", grayimage->coding, grayimage->n_planes);
		}
	
		if(savepgm)
			grayimage->save("gray.pgm");
	}
	
	delete grayimage;

	return(0);
}
