#ifndef MATRIX2D_INC
#define MATRIX2D_INC

#include "general.h"
#include "XFile.h"
#include "matrix.h"

namespace Torch {

/** This class is designed to handle a 2D matrix for geometric operations

    @author Sebastien Marcel (marcel@idiap.ch)
    @author Dan Sunday (http://www.softsurfer.com/)
    @version 2.0
    \Date
    @since 1.0
*/
class Matrix2D : public Mat
{
	char buf_sprint[250];
public:

	/** @name constructors */
	//@{ 
	/// creates an zero 2D matrix
	Matrix2D();
	
	/** creates a 2D matrix from coefficients.
	 
	    the 2D matrix will be:
	    \begin{verbatim}
		[ a b 
		  c d ]
	    \end{verbatim}
	*/
	Matrix2D(double a, double b, double c, double d);

	/// creates a 2D matrix from another one
	Matrix2D(const Matrix2D &);
	//@}
	
	///
	~Matrix2D();

	//-----
	
	/// save the matrix
	void saveXFile(XFile *file);

	/// load a matrix
	void loadXFile(XFile *file);
	
	/// print the matrix into a string
	const char *sprint();

	//-----
	
	/// copy a matrix
	Matrix2D operator=(Matrix2D);

	//-----

	/// unary minus operation
	Matrix2D operator-();
	
	//-----
	
	/** @name scalar multiplication */
	//@{
	///
	friend Matrix2D operator*(int, Matrix2D);
	
	///
	friend Matrix2D operator*(double, Matrix2D);

	///
	friend Matrix2D operator*(Matrix2D, int);

	///
	friend Matrix2D operator*(Matrix2D, double);
	//@}
	
	/** @name scalar division */
	//@{
	///
	friend Matrix2D operator/(Matrix2D, int);

	///
	friend Matrix2D operator/(Matrix2D, double);
	//@}
	
	//-----
	
	/** @name Matrix2D arithmetic operations */
	//@{
	/// addition
	Matrix2D operator+(Matrix2D);
	
	/// substraction
	Matrix2D operator-(Matrix2D);
	
	/// scalar multiplication
	Matrix2D& operator*=(double);
	
	/// scalar division
	Matrix2D& operator/=(double);
	
	/// increment
	Matrix2D& operator+=(Matrix2D);
	
	/// decrement
	Matrix2D& operator-=(Matrix2D);
	//@}
};

}

#endif
