#ifndef HAAR_STUMP_MACHINE_INC
#define HAAR_STUMP_MACHINE_INC

#include "StumpMachine.h"
#include "ipHaar.h"

namespace Torch {

/** This class is designed to train/support weak classifiers based on haar-like features

    @author Yann Rodiguez (rodrig@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class HaarStumpMachine : public StumpMachine
{   
protected:
	// Size of the pattern where the mask behave
	int width_pattern;
	int height_pattern;

	//
	real *integralimage2;
	
public:
   	/// The current mask to use to compute the feature
	ipHaar *the_mask;

	/// All the possible masks (in training mode only)
   	ipHaar **masks;

	//-----
	
	/// constructor for training
	HaarStumpMachine(int n_features_, ipHaar **masks_);

	/// constructor for testing
	HaarStumpMachine(int width_pattern_, int height_pattern_, real *integralimage2_ = NULL);

	/// decode the parameters of the machine
	virtual void setParams(int feature_id_, real threshold_, int direction_);

	/// compute the feature #feature_id_# for the sequence #inputs_# 
        virtual real getFeature(int feature_id_, Sequence *inputs_);

	/// load the parameters of the machine
	virtual void loadXFile(XFile *file);

	/// save the parameters of the machine
	virtual void saveXFile(XFile *file);

	///	
	virtual ~HaarStumpMachine();
};

}

#endif
