#include "Mahanalobis.h"

namespace Torch {

//
mMahanalobis::mMahanalobis(int n_inputs_, EigenMachine *eigen_machine_) : Metric(n_inputs_)
{
	eigen_machine = eigen_machine_;

	n_outputs = eigen_machine->n_outputs;
	   
   	m = (real *) allocator->alloc(n_outputs*sizeof(real));
   	n = (real *) allocator->alloc(n_outputs*sizeof(real));
   	stdv = (real *) allocator->alloc(n_outputs*sizeof(real));

	for(int i = 0 ; i < n_outputs ; i++)
	{
		m[i] = 0.0;
		n[i] = 0.0;
		stdv[i] = sqrt(eigen_machine->eigenvalues[i]);
	}
	
	seq = new(allocator) Sequence(1, n_inputs);
}
	
void mMahanalobis::projection(real *sample_, real *template_)
{
	seq->copyFrom(sample_);
	eigen_machine->forward(seq);
	for(int i = 0 ; i < n_outputs ; i++) m[i] = eigen_machine->outputs->frames[0][i] / stdv[i];

	seq->copyFrom(template_);
	eigen_machine->forward(seq);
	for(int i = 0 ; i < n_outputs ; i++) n[i] = eigen_machine->outputs->frames[0][i] / stdv[i];
}

//
mMahanalobisL1::mMahanalobisL1(int n_inputs_, EigenMachine *eigen_machine_) : mMahanalobis(n_inputs_, eigen_machine_)
{
}
	
real mMahanalobisL1::measure(real *sample_, real *template_)
{
	//
	projection(sample_, template_);

	//
	real d_ = 0.0;

	for(int i = 0 ; i < n_outputs ; i++)
	{
		real z = m[i] - n[i];
		d_ += fabs(z);
	}
	
	return d_;
}

//
mMahanalobisL2::mMahanalobisL2(int n_inputs_, EigenMachine *eigen_machine_) : mMahanalobis(n_inputs_, eigen_machine_)
{
	eigen_machine = eigen_machine_;
}
	
real mMahanalobisL2::measure(real *sample_, real *template_)
{
	//
	projection(sample_, template_);

	//
	real d_ = 0.0;

	for(int i = 0 ; i < n_outputs ; i++)
	{
		real z = m[i] - n[i];
		d_ += z * z;
	}
	
	return sqrt(d_);
}

//
mMahanalobisCosine::mMahanalobisCosine(int n_inputs_, EigenMachine *eigen_machine_) : mMahanalobis(n_inputs_, eigen_machine_)
{
}
	
real mMahanalobisCosine::measure(real *sample_, real *template_)
{
	//
	projection(sample_, template_);

	//
	real a, b, c;

	a = scalarProduct(n_inputs, m, n);
	b = scalarProduct(n_inputs, m, m);
	c = scalarProduct(n_inputs, n, n);
	   
	return -a / sqrt(b) * sqrt(c);
}

}
