#ifndef METRIC_INC
#define METRIC_INC

#include "Object.h"

namespace Torch {

/// This function computes the scalar product between two vectors
real scalarProduct(int n_inputs, real *x, real *y);

/** This class is designed to compute metrics

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class Metric : public Object
{
public:
	/// verbose
	bool verbose;

	/// number of inputs
	int n_inputs;
	
	//-----
	
	///
	Metric(int n_inputs_);

	///
	virtual real measure(real *sample_, real *template_ = NULL) = 0;

	///
	virtual ~Metric() {};
};

/** This class is designed to compute the CityBlock distance (also called L1 norm)

    \begin{eqnarray}
    d(X,Y) = \sum_i |X_i - Y_i|
    \end{eqnarray}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mCityBlock : public Metric
{
public:
	//-----
	
	///
	mCityBlock(int n_inputs_);

	///
	virtual real measure(real *sample_, real *template_ = NULL);

	///
	virtual ~mCityBlock() {};
};

/** This class is designed to compute the Euclidean distance (also called L2 norm)

    \begin{eqnarray}
    d(X,Y) = \sqrt{ \sum_i (X_i - Y_i)^2
    \end{eqnarray}

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mEuclidean : public Metric
{
public:
	//-----
	
	///
	mEuclidean(int n_inputs_);

	///
	virtual real measure(real *sample_, real *template_ = NULL);

	///
	virtual ~mEuclidean() {};
};

}

#endif

