#include "NonOrientedPatternFusion.h"
	
namespace Torch {

NonOrientedPatternFusion::NonOrientedPatternFusion(PatternMerger *merger_, int view_, int max_patterns_)
{
	addBOption("verbose", &verbose, false, "verbose");

	max_patterns = max_patterns_;
	merger = merger_;
	
	view = view_;
	
	patterns = NULL;
	merged_patterns = NULL;
	TableOverlap = NULL;
	
	n_patterns = 0;
	n_merged_patterns = 0;
	
	patterns = (Pattern **)malloc(sizeof(Pattern *) * max_patterns);
	for(int i = 0 ; i < max_patterns ; i++) patterns[i] = new Pattern();

	merged_patterns = (Pattern **)malloc(sizeof(Pattern *) * max_patterns);
	for(int i = 0 ; i < max_patterns ; i++) merged_patterns[i] = new Pattern();

	TableOverlap = new sTableOverlap [max_patterns];
	if(TableOverlap == NULL)
		print("Impossible to allocate %d elements for TableOverlap.\n", max_patterns);

	for(int i = 0 ; i < max_patterns ; i++)
	{
		TableOverlap[i].n_overlap = 0;
		TableOverlap[i].n_real_overlap = 0;
		TableOverlap[i].surf = 0;

		TableOverlap[i].overlap = new sSubTableOverlap [max_patterns];
		if(TableOverlap[i].overlap == NULL)
		{
			for(int j = 0 ; j < i ; j++) delete [] TableOverlap[j].overlap;

			delete [] TableOverlap;
			TableOverlap = NULL;

			print("Impossible to allocate sub-element for TableOverlap.\n");
		}
	}
}

void NonOrientedPatternFusion::reset()
{
	n_patterns = 0;
}

int NonOrientedPatternFusion::IsInTagOverlap(int index, int N, sSubTableOverlap *suboverlap)
{
	for(int i = 0 ; i < N ; i++)
		if(suboverlap[i].tag == true)	 
			if(suboverlap[i].index == index) return i;

	return -1;
}

int NonOrientedPatternFusion::IsInOverlap(int index, int N, sSubTableOverlap *suboverlap)
{
	for(int i = 0 ; i < N ; i++)
	   	if(suboverlap[i].index == index) return i;

	return -1;
}
	
bool NonOrientedPatternFusion::doTableOverlap(int N, Pattern **patterns_, float surfoverlap, bool useInclusion)
{
	int n;
	float s;

	// For each pattern
	for(int i = 0 ; i < N ; i++)
	{
		n = 0;

		// Compute the overlapping surface with other patterns
		for(int j = 0 ; j < N ; j++)
			if(i != j)
			{
				s = SurfOverlap(patterns_[i], patterns_[j]);

				if(useInclusion)
					if(rectIsInRect(patterns_[i], patterns_[j]) == true) s = 0.0;
				   
				//if(s >= surfoverlap)
				if(s > surfoverlap)
				{
					TableOverlap[i].overlap[n].index = j;
					TableOverlap[i].overlap[n].surf = s;
					TableOverlap[i].overlap[n].tag = true;

					n++;
				}
			}
		
		TableOverlap[i].n_overlap = n;
		TableOverlap[i].n_real_overlap = n;
		TableOverlap[i].surf = patterns_[i]->w * patterns_[i]->h;
	}

	if(verbose)
	{
	   	// Print the table
		
		print("Overlap table :\n");
		printTableOverlap(N);
		/*
		for(int i = 0 ; i < N ; i++)
		{
			print("[%d %d] (%d|%d) = [ ", i, TableOverlap[i].surf, TableOverlap[i].n_overlap, TableOverlap[i].n_real_overlap);
	   
			for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
				print("(%d %2.3f %d) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf, TableOverlap[i].overlap[j].tag);

			print("]\n");
		}
		*/
	}

	return true;
}

void NonOrientedPatternFusion::printTableOverlap(int N)
{
	for(int i = 0 ; i < N ; i++)
	{
		print("[%d %d] (%d|%d) = [ ", i, TableOverlap[i].surf, TableOverlap[i].n_overlap, TableOverlap[i].n_real_overlap);
	   
		for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
			print("(%d %2.3f %d) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf, TableOverlap[i].overlap[j].tag);

		print("]\n");
	}
}

bool NonOrientedPatternFusion::tagTableOverlap(int N, Pattern **patterns_)
{
	int c;
	
	// For each pattern
	for(int i = 0 ; i < N ; i++)
	{
	   	// If the pattern i overlaps other patterns
	   	if(TableOverlap[i].n_real_overlap != 0)
	   	{
		   	//
		   	// Prepare the list of patterns overlapping pattern i
	      		sSubTableOverlap *overlaps_ = TableOverlap[i].overlap;

			//
			// For each pattern
	      		for(int j = 0 ; j < N ; j++)
			{
			   	// If pattern j is not pattern i and pattern j overlaps other patterns
		 		if((i != j) && (TableOverlap[j].n_real_overlap != 0))
		 		{
				   	//
				   	// For each patterns from the list of patterns overlapping pattern i
		    			for(int k = 0 ; k < TableOverlap[i].n_overlap ; k++)
		    			{
					   	// Check if a pattern from the list of patterns overlapping pattern i
						// is also overlapping a pattern from the list of patterns overlapping pattern j
		    				if(overlaps_[k].tag == true)
						{
			   				c = IsInTagOverlap(overlaps_[k].index, TableOverlap[j].n_overlap, TableOverlap[j].overlap);

			   				if(c != -1)
			   				{
			      					TableOverlap[j].n_real_overlap--;
			      					TableOverlap[j].overlap[c].tag = false;
			   				}
						}
		    			}
		 
		    			c = IsInTagOverlap(i, TableOverlap[j].n_overlap, TableOverlap[j].overlap);

		    			if(c != -1)			   
		    			{
		       				TableOverlap[j].n_real_overlap--;
		       				TableOverlap[j].overlap[c].tag = false;
		    			}
		 		}
			}
	   	} // End if (TableOverlap[i].n_real_overlap != 0)  
	}

	//
	// Checking if no more overlaps
	bool bTable;
	
	bTable = true;
	
	if(verbose) print("Overlap table after tagging:\n");
	
	for(int i = 0 ; i < N ; i++)
	{	
	   	if((TableOverlap[i].n_overlap != 0) 	
			&& (TableOverlap[i].n_overlap != TableOverlap[i].n_real_overlap)) bTable = false;
		
	   	if(verbose)
	   	{
	      		print("[%d] (%d|%d) = [ ", i, TableOverlap[i].n_overlap, TableOverlap[i].n_real_overlap);
	   		   
	      		for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)		      
		 		print("(%d %2.3f %d) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf, TableOverlap[i].overlap[j].tag);   
	   	}	  

	   	if(verbose) print("]\n");
	}
	
	if(verbose)
	{
		if(bTable == true) print("   + The overlap table is cleaned\n");
		else print("   + The overlap table is not fully resolved\n");
	}
	
	return bTable;
}

bool NonOrientedPatternFusion::resolveTableOverlap(int N, Pattern **patterns_)
{
	int c;
	
	float max_;
	int argmax_;

	for(int i = 0 ; i < N ; i++)
	{
	   	if((TableOverlap[i].n_overlap != 0) && (TableOverlap[i].n_overlap != TableOverlap[i].n_real_overlap))
	   	{
	      		sSubTableOverlap *overlaps_ = TableOverlap[i].overlap;

	      		for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)	
	      		{
		 		if(overlaps_[j].tag == true)
		 		{
		    			max_ = overlaps_[j].surf;
		    			argmax_ = j;

		    			for(int k = 0 ; k < TableOverlap[i].n_overlap ; k++)
		    			{		 
		       				if(overlaps_[k].tag == false)
			  				if(overlaps_[k].surf > max_)
			  				{		       		    
			     					max_ = overlaps_[k].surf;			    
			     					argmax_ = k;
			  				}
		    			}

		    			if(argmax_ == j)
		    			{
		    				TableOverlap[i].n_real_overlap--;
		    				overlaps_[j].tag = false;
		    			}
		    
		    			c = IsInOverlap(i, TableOverlap[overlaps_[argmax_].index].n_overlap, TableOverlap[overlaps_[argmax_].index].overlap);

		    			if(c != -1)			   
		    			{
		       				TableOverlap[overlaps_[argmax_].index].n_real_overlap++;
		       				TableOverlap[overlaps_[argmax_].index].overlap[c].tag = true;
		    			}
		 		}
	      		}
	   	} // End of ((TableOverlap[i].n_overlap != 0) && (TableOverlap[i].n_overlap != TableOverlap[i].n_real_overlap))
	}

	if(verbose)
	{
	   	// Print the table
		
		print("Overlap table after resolving:\n");
	
		for(int i = 0 ; i < N ; i++)
		{		   	
			print("[%d] (%d|%d) = [ ", i, TableOverlap[i].n_overlap, TableOverlap[i].n_real_overlap);
	   
			for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
				print("(%d %2.3f %d) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf, TableOverlap[i].overlap[j].tag);
	
			print("]\n");
		}
	}
		
	return true;
}

struct int_surf
{
	int index;
	real surf;   
};

int sortIndexedTableOverlapDec(const void *e1, const void *e2)
{
	int_surf *is1, *is2;
  
	is1 = (int_surf *) e1;
	is2 = (int_surf *) e2;

	if((is1->surf) < (is2->surf)) return 1;
	else if((is1->surf) > (is2->surf)) return -1;
	else return 0;
}

bool NonOrientedPatternFusion::doOverlap(int N, Pattern **patterns_, float surfoverlap, float threshold_overlap, bool computeActivation)
{
	int iCadre, jCadre;
	bool bTable;

	if(N == 0) return false;
	if(N > max_patterns) return false;
	
	// Init tables
	n_patterns = 0;
	n_merged_patterns = 0;

	for(int i = 0 ; i < N ; i++)
	{
		TableOverlap[i].n_overlap = 0;
		TableOverlap[i].n_real_overlap = 0;

		// Remove non-zero view
		if((patterns_[i]->view == view) && (patterns_[i]->confidence >= threshold_overlap))
			merged_patterns[n_merged_patterns++]->copyFrom(patterns_[i]);
	}

	if(verbose)
	{
		print(" + Overlapping %d patterns over %d at view %d with %g \% ...\n", n_merged_patterns, N, view, surfoverlap * 100.0);

		if(computeActivation)
			print(" + Compute Activation\n");
	}

	if(n_merged_patterns == 0) return true;
	
	// Sort the patterns
 	qsort(merged_patterns, n_merged_patterns, sizeof(Pattern *), sortPatternConfDec);

	// Compute the overlap table
	doTableOverlap(n_merged_patterns, merged_patterns, surfoverlap);

	// Tag overlaps to merge
	bTable = tagTableOverlap(n_merged_patterns, merged_patterns);

	// If the table is fully resolved (no overlaps)
	if(bTable == true)
	{
		if(verbose) message("1-pass ...");

	   	n_patterns = 0;

	   	for(int i = 0 ; i < n_merged_patterns ; i++)
	      		if(TableOverlap[i].n_overlap == 0)
			{
				patterns[n_patterns]->copyFrom(merged_patterns[i]); 
				patterns[n_patterns]->activation = 1.0;
				n_patterns++;
			}
	
		if(verbose)
	   		print("   + NewPatterns = %d\n", n_patterns);

		// Sort the patterns
		if(computeActivation)
	   		qsort(patterns, n_patterns, sizeof(Pattern *), sortPatternActivDec);
		else qsort(patterns, n_patterns, sizeof(Pattern *), sortPatternConfDec);
	}
	else
	{
		if(verbose) message("multi-pass ...");

		while(bTable == false)
		{	
	   		resolveTableOverlap(n_merged_patterns, merged_patterns);
	   
	   		n_patterns = 0;

	   		for(int i = 0 ; i < n_merged_patterns ; i++)
	   		{
	      			if(TableOverlap[i].n_overlap == 0) patterns[n_patterns++]->copyFrom(merged_patterns[i]); 
		      		else if(TableOverlap[i].n_real_overlap != 0)
	      			{
					iCadre = i;

					merger->reset();
				
					merger->add(merged_patterns[iCadre]);
					if(verbose)
						merged_patterns[iCadre]->sprint();

		 			for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
					{
		    				if(TableOverlap[i].overlap[j].tag == true)	
		    				{
		      					jCadre = TableOverlap[i].overlap[j].index;
					
							merger->add(merged_patterns[jCadre]);
							if(verbose) 
								merged_patterns[jCadre]->sprint();
		    				}
					}
				
					merger->merge(patterns[n_patterns]);
					if(computeActivation) patterns[n_patterns]->activation = (real) merger->n_patterns - 1.0;
					n_patterns++;
	      			}
	   		}

			if(verbose)
	   			print("   + NewPatterns = %d\n", n_patterns);

			//
	   		qsort(patterns, n_patterns, sizeof(Pattern *), sortPatternConfDec);

			//
	   		doTableOverlap(n_patterns, patterns, surfoverlap);
	   
			//
	   		bTable = tagTableOverlap(n_patterns, patterns);
	 
			if(bTable == false)
			{
				n_merged_patterns = n_patterns;
		   		for(int i = 0 ; i < n_merged_patterns ; i++) merged_patterns[i]->copyFrom(patterns[i]);
			}		
		} // End of while (bTable == false)
	   
		//
		if(computeActivation)
		{
			for(int i = 0 ; i < n_patterns ; i++) patterns[i]->activation += 1.0;

			int_surf *indexTableOverlap = new int_surf [n_patterns];
			for(int i = 0 ; i < n_patterns ; i++)
			{
				TableOverlap[i].n_overlap = 0;
				TableOverlap[i].n_real_overlap = 0;
			}

			//
			qsort(patterns, n_patterns, sizeof(Pattern *), sortPatternActivDec);

			// Compute the overlap table
			doTableOverlap(n_patterns, patterns, 0.01, true);

			for(int i = 0 ; i < n_patterns ; i++)
			{
				indexTableOverlap[i].index = i;
				indexTableOverlap[i].surf = TableOverlap[i].surf;
			}

			//
			qsort(indexTableOverlap, n_patterns, sizeof(int_surf), sortIndexedTableOverlapDec);
	
			//
			if(verbose)
			{
				print("Sorted overlap table :\n");
			
				for(int j = 0 ; j < n_patterns ; j++)
				{
					int i = indexTableOverlap[j].index;
			
					print("[%d %d] (%d|%d) = [ ", i, TableOverlap[i].surf, TableOverlap[i].n_overlap, TableOverlap[i].n_real_overlap);
	   
					for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
						print("(%d %2.3f %d) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf, TableOverlap[i].overlap[j].tag);

					print("]\n");
				}
			}

			int n_entries = n_patterns;

			while(n_entries != 0)
			{
			   n_entries = n_patterns;

	   		   for(int p = 0 ; p < n_patterns ; p++)
	   		   {
				int i = indexTableOverlap[p].index;

	      			if(TableOverlap[i].n_real_overlap != 0)
	     			{
					if(verbose)
					{
						print("Searching for overlaps into (%d):", i);
						patterns[i]->sprint();
					}

					//real min_ = patterns[i]->activation;
					real min_ = patterns[i]->confidence;
					int argmin_ = i;
					int argargmin_ = -1;

	 				for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
					{
	    					if(TableOverlap[i].overlap[j].tag == true)	
	    					{
	      						int k = TableOverlap[i].overlap[j].index;

							//if(patterns[k]->activation < min_)
							if(patterns[k]->confidence < min_)
							{
								//min_ = patterns[k]->activation;
								min_ = patterns[k]->confidence;
								argmin_ = k;
								argargmin_ = j;
							}
						}
					}
	
					if(verbose)
					{
						print("min found (%d): ", argmin_);
						patterns[argmin_]->sprint();
					}

					//
					patterns[argmin_]->activation = 0.0;

					//
					if(argmin_ == i)
					{
				   		TableOverlap[i].n_real_overlap = 0;
					
						// remove i from the lists
	 					for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
						{
	    						if(TableOverlap[i].overlap[j].tag == true)	
	    						{
	      							int k = TableOverlap[i].overlap[j].index;
								int r = IsInTagOverlap(i, TableOverlap[k].n_overlap, TableOverlap[k].overlap);

								if(r != -1)
								{
									TableOverlap[k].overlap[r].tag = false;
									TableOverlap[k].n_real_overlap--;
								}
							      
	    						}
						}
					}
					else
					{
						TableOverlap[i].n_real_overlap--;
						TableOverlap[i].overlap[argargmin_].tag = false;
					}
      				}
				else n_entries--;
			   }
   			}
		
			//
			if(verbose)
			{
				print("Indexed overlap table :\n");
				for(int j = 0 ; j < n_patterns ; j++)
				{
					int i = indexTableOverlap[j].index;
			
					print("[%d %d] (%d|%d) = [ ", i, TableOverlap[i].surf, TableOverlap[i].n_overlap, TableOverlap[i].n_real_overlap);
	   
					for(int j = 0 ; j < TableOverlap[i].n_overlap ; j++)
						if(TableOverlap[i].overlap[j].tag)
							print("(%d %2.3f 1) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf);
						else print("(%d %2.3f 0) ", TableOverlap[i].overlap[j].index, TableOverlap[i].overlap[j].surf);

					print("]\n");
				}
			}
		
			delete [] indexTableOverlap;

			//
			qsort(patterns, n_patterns, sizeof(Pattern *), sortPatternActivDec);
		}
		else for(int i = 0 ; i < n_patterns ; i++) patterns[i]->activation = 1.0;
	}

	return true;
}

NonOrientedPatternFusion::~NonOrientedPatternFusion()
{
	for(int i = 0 ; i < max_patterns ; i++)
	{
		delete patterns[i];
		delete merged_patterns[i];
	}
	free(patterns);
	free(merged_patterns);
	   
	if(TableOverlap != NULL)
	{
		for(int j = 0 ; j < max_patterns ; j++) delete [] TableOverlap[j].overlap;

		delete [] TableOverlap;
		TableOverlap = NULL;
	}
}

}

