#include "StumpMachine.h"
#include "XFile.h"

namespace Torch {

StumpMachine::StumpMachine(int n_features_)
{
	feature_id = -1;
	threshold = 0.;
	direction = 0;

	//
        outputs = new(allocator) Sequence(1, 1);

        //
        n_features = n_features_;
}

void StumpMachine::setParams(int feature_id_, real threshold_, int direction_)
{
	feature_id = feature_id_;
	threshold = threshold_;
	direction = direction_;
}
        
real StumpMachine::getFeature(int feature_id_, Sequence *inputs_)
{
	return inputs_->frames[0][feature_id_]; 
}

void StumpMachine::forward(Sequence *inputs)
{
	if(feature_id == -1)
		error("StumpMachine::forward(): feature_id is not defined.");

	// computes the value of the feature #feature_id#
	real feature = getFeature(feature_id, inputs);

	if(direction == 1)
	{
		if(feature > threshold) outputs->frames[0][0] = 1;
		else outputs->frames[0][0] = -1;
	}
	else
	{
		if(feature < threshold) outputs->frames[0][0] = 1;
		else outputs->frames[0][0] = -1;
	}	
}

void StumpMachine::loadXFile(XFile *file)
{
   	file->taggedRead(&feature_id, sizeof(int), 1, "FEATURE");
	file->taggedRead(&threshold, sizeof(real), 1, "THRESHOLD");
	file->taggedRead(&direction, sizeof(int), 1, "DIR");
}

void StumpMachine::saveXFile(XFile *file)
{
   	file->taggedWrite(&feature_id, sizeof(int), 1, "FEATURE");
	file->taggedWrite(&threshold, sizeof(real), 1, "THRESHOLD");
	file->taggedWrite(&direction, sizeof(int), 1, "DIR");
}

StumpMachine::~StumpMachine()
{
}

}
