#include "ipHaar.h"
#include "XFile.h"

namespace Torch {

ipHaar::ipHaar(int width_pattern_, int height_pattern_, int mask_id_, int x_, int y_, int w_, int h_) : ipCore("gray")
{
	width_pattern = width_pattern_;
	height_pattern = height_pattern_;

	width_image = width_pattern_;
        height_image = height_pattern_;

	mask_id = mask_id_;

	x_mask = x_;
	y_mask = y_;
	w_mask = w_;
	h_mask = h_;

	x_window = 0;
	y_window = 0;
	w_window = width_pattern;
	h_window = height_pattern;

	x_mask_1 = x_mask - 1;
	y_mask_1 = y_mask - 1;

	scale_x = 1.0;
	scale_y = 1.0;
	scale_xy_inv = 1.0;

	x = x_mask_1;
	y = y_mask_1;
	w = w_mask;
	h = h_mask;

	w2 = w/2;
	h2 = h/2;

	w3 = w/3;
	h3 = h/3;

	w3_2 = 2*w3;
	h3_2 = 2*h3;

	integralimage2 = NULL;

	seq_out = new(allocator)Sequence(1, 1);
}

void ipHaar::resetImage(int width_image_, int height_image_)
{
	width_image = width_image_;
	height_image = height_image_;
}

void ipHaar::resetWindowX(int x_window_)
{
	x_window = x_window_;
	
	x = x_window + (int) (x_mask * scale_x + 0.5) - 1;
}

void ipHaar::resetWindowY(int y_window_)
{
	y_window = y_window_;

	y = y_window + (int) (y_mask * scale_y + 0.5) - 1;
}

void ipHaar::resetWindowXY(int x_window_, int y_window_)
{
	if(x_window != x_window_)
	{
		x_window = x_window_;
	
		x = x_window + (int) (x_mask * scale_x + 0.5) - 1;
	}
	
	if(y_window != y_window_)
	{
		y_window = y_window_;
	
		y = y_window + (int) (y_mask * scale_y + 0.5) - 1;
	}
}

void ipHaar::resetWindowSize(int w_window_, int h_window_)
{
	if(w_window != w_window_)
	{
		w_window = w_window_;
		
		scale_x = (real) w_window / width_pattern;

		w = (int) (w_mask * scale_x + 0.5);
		w2 = w/2;
		w3 = w/3;
		w3_2 = 2*w3;
	}

	if(h_window != h_window_)
	{
		h_window = h_window_;
		
		scale_y = (real) h_window / height_pattern;

		h = (int) (h_mask * scale_y + 0.5);
		h2 = h/2;
		h3 = h/3;
		h3_2 = 2*h3;
	}
	
	scale_xy_inv = 1.0 / (scale_x * scale_y);
}

real ipHaar::p1(Sequence *seq_in)
{
	int X, Y;

	X = x;	
	Y = y;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p2(Sequence *seq_in)
{
	int X, Y;

	X = x + w2;	
	Y = y;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p3(Sequence *seq_in)
{
	int X, Y;

	X = x + w;	
	Y = y;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p4(Sequence *seq_in)
{
	int X, Y;

	X = x;	
	Y = y + h2;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p5(Sequence *seq_in)
{
	int X, Y;

	X = x + w2;	
	Y = y + h2;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p6(Sequence *seq_in)
{
	int X, Y;

	X = x + w;	
	Y = y + h2;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p7(Sequence *seq_in)
{
	int X, Y;

	X = x;	
	Y = y + h;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p8(Sequence *seq_in)
{
	int X, Y;

	X = x + w2;	
	Y = y + h;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p9(Sequence *seq_in)
{
	int X, Y;

	X = x + w;	
	Y = y + h;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p12(Sequence *seq_in)
{
	int X, Y;

	X = x + w3;	
	Y = y;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p23(Sequence *seq_in)
{
	int X, Y;

	X = x + w3_2;	
	Y = y;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p14(Sequence *seq_in)
{
	int X, Y;

	X = x;	
	Y = y + h3;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p36(Sequence *seq_in)
{
	int X, Y;

	X = x + w;	
	Y = y + h3;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p47(Sequence *seq_in)
{
	int X, Y;

	X = x;	
	Y = y + h3_2;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p69(Sequence *seq_in)
{
	int X, Y;

	X = x + w;	
	Y = y + h3_2;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p78(Sequence *seq_in)
{
	int X, Y;

	X = x + w3;	
	Y = y + h;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p89(Sequence *seq_in)
{
	int X, Y;

	X = x + w3_2;	
	Y = y + h;	

	return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p15(Sequence *seq_in)
{
        int X, Y;

        X = x + w3;
        Y = y + h3;

        return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p35(Sequence *seq_in)
{
        int X, Y;

        X = x + w3_2;
        Y = y + h3;

        return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p75(Sequence *seq_in)
{
        int X, Y;
	
        X = x + w3;
        Y = y + h3_2;
	
        return seq_in->frames[0][X + Y * width_image];
}

real ipHaar::p95(Sequence *seq_in)
{
        int X, Y;

        X = x + w3_2;
        Y = y + h3_2;
	
        return seq_in->frames[0][X + Y * width_image];
}

extern real iisum_check_limit(real *, int, int, int, int, int, int);
extern real iisum(real *, int, int, int, int, int, int);

void ipHaar::process(Sequence *seq_in)
{
	real norm = 1.0;

	if(integralimage2 != NULL)
	{
		real N_ = w_window * h_window;
		
		real z = iisum_check_limit(seq_in->frames[0], width_image, height_image, x_window, y_window, w_window, h_window) / N_;

		norm = iisum_check_limit(integralimage2, width_image, height_image, x_window, y_window, w_window, h_window) / N_ - z*z;
		
		//real z = iisum(seq_in->frames[0], width_image, height_image, x_window, y_window, w_window, h_window) / N_;
		//real norm = iisum(integralimage2, width_image, height_image, x_window, y_window, w_window, h_window) / N_ - z*z;
		
	}
	
	seq_out->frames[0][0] = applyMask(seq_in) * scale_xy_inv / norm;
}

void ipHaar::loadXFile(XFile *file)
{
}

void ipHaar::saveXFile(XFile *file)
{
        file->taggedWrite(&mask_id, sizeof(int), 1, "MASK_ID");
        file->taggedWrite(&x_mask, sizeof(int), 1, "X_MASK");
        file->taggedWrite(&y_mask, sizeof(int), 1, "Y_MASK");
        file->taggedWrite(&w_mask, sizeof(int), 1, "W_MASK");
        file->taggedWrite(&h_mask, sizeof(int), 1, "H_MASK");
}

void ipHaar::info()
{
	print("ipHaar::info()\n");
	print(" + mask id = %d\n", mask_id);
	print(" + x_mask = %d\n", x_mask);
	print(" + y_mask = %d\n", y_mask);
	print(" + w_mask = %d\n", w_mask);
	print(" + h_mask = %d\n", h_mask);
	print(" + width pattern = %d\n", width_pattern);
	print(" + height pattern = %d\n", height_pattern);
	print(" + x = %d\n", x);
	print(" + y = %d\n", y);
	print(" + w = %d\n", w);
	print(" + h = %d\n", h);
	print(" + width image = %d\n", width_image);
	print(" + height image = %d\n", height_image);
	print(" + x_window = %d\n", x_window);
	print(" + y_window = %d\n", y_window);
	print(" + w_window = %d\n", w_window);
	print(" + h_window = %d\n", h_window);
	print(" + scale x = %g\n", scale_x);
	print(" + scale y = %g\n", scale_y);
	print(" + scale_xy_inv = %g\n", scale_xy_inv);
}

ipHaar::~ipHaar()
{
}

// Bi-Vertical Haar feature
biVertHaar::biVertHaar(int width_pattern_, int height_pattern_, int x_, int y_, int w_, int h_) : ipHaar(width_pattern_ , height_pattern_, 1, x_, y_, w_, h_)
{
}

real biVertHaar::applyMask(Sequence *seq_in)
{
	real P1, P2, P3, P7, P8, P9;

	P1 = p1(seq_in);
	P2 = p2(seq_in);
	P3 = p3(seq_in);
	P7 = p7(seq_in);
	P8 = p8(seq_in);
	P9 = p9(seq_in);

	real feature = 2.0 * P8 + P1 - 2.0 * P2 - P7 - P9 + P3;

	return feature;
}

// Bi-Horizonal Haar feature
biHorizHaar::biHorizHaar(int width_pattern_, int height_pattern_, int x_, int y_, int w_, int h_) : ipHaar(width_pattern_ , height_pattern_, 2, x_, y_, w_, h_)
{
}

real biHorizHaar::applyMask(Sequence *seq_in)
{
	real P1, P3, P4, P6, P7, P9;

	P1 = p1(seq_in);
	P3 = p3(seq_in);
	P4 = p4(seq_in);
	P6 = p6(seq_in);
	P7 = p7(seq_in);
	P9 = p9(seq_in);

	real feature = 2.0 * P6 + P1 - P3 - 2.0 * P4 - P9 + P7;

	return feature;
}

// Diagonal Haar feature
diagHaar::diagHaar(int width_pattern_, int height_pattern_, int x_, int y_, int w_, int h_) : ipHaar(width_pattern_ , height_pattern_, 3, x_, y_, w_, h_)
{
}

real diagHaar::applyMask(Sequence *seq_in)
{
	real P1, P2, P3, P4, P5, P6, P7, P8, P9;

	P1 = p1(seq_in);
	P2 = p2(seq_in);
	P3 = p3(seq_in);
	P4 = p4(seq_in);
	P5 = p5(seq_in);
	P6 = p6(seq_in);
	P7 = p7(seq_in);
	P8 = p8(seq_in);
	P9 = p9(seq_in);

	real feature = P1 - 2.0 * P2 + P3 - 2.0 * P4 + 4.0 * P5 - 2.0 * P6 + P7 - 2.0 * P8 + P9;

	return feature;
}

// Tri-Vertical Haar feature
triVertHaar::triVertHaar(int width_pattern_, int height_pattern_, int x_, int y_, int w_, int h_) : ipHaar(width_pattern_ , height_pattern_, 4, x_, y_, w_, h_)
{
}

real triVertHaar::applyMask(Sequence *seq_in)
{
	real P1, P3, P7, P9;
	real P89, P12, P23, P78;

	P1 = p1(seq_in);
	P3 = p3(seq_in);
	P7 = p7(seq_in);
	P9 = p9(seq_in);

	P89 = p89(seq_in);
	P12 = p12(seq_in);
	P23 = p23(seq_in);
	P78 = p78(seq_in);

	real feature = P9 + P1 - P3 - P7 - 3.0 * (P89 + P12 - P23 - P78);

	return feature;
}

// Tri-Horizonal Haar feature
triHorizHaar::triHorizHaar(int width_pattern_, int height_pattern_, int x_, int y_, int w_, int h_) : ipHaar(width_pattern_ , height_pattern_, 5, x_, y_, w_, h_)
{
}

real triHorizHaar::applyMask(Sequence *seq_in)
{
	real P1, P3, P7, P9;
	real P69, P14, P36, P47;

	P1 = p1(seq_in);
	P3 = p3(seq_in);
	P7 = p7(seq_in);
	P9 = p9(seq_in);

	P69 = p69(seq_in);
	P14 = p14(seq_in);
	P36 = p36(seq_in);
	P47 = p47(seq_in);

	real feature = P9 + P1 - P3 - P7 - 3.0 * (P69 + P14 - P36 - P47);

	return feature;
}

// Central Haar feature
centralHaar::centralHaar(int width_pattern_, int height_pattern_, int x_, int y_, int w_, int h_) : ipHaar(width_pattern_ , height_pattern_, 5, x_, y_, w_, h_)
{
}

real centralHaar::applyMask(Sequence *seq_in)
{
        real P1, P3, P7, P9;
        real P95, P15, P35, P75;

        P1 = p1(seq_in);
        P3 = p3(seq_in);
        P7 = p7(seq_in);
        P9 = p9(seq_in);

	P95 = p95(seq_in);
        P15 = p15(seq_in);
	P35 = p35(seq_in);
	P75 = p75(seq_in);
	
	real feature = P9 + P1 - P3 - P7 - 9.0 * (P95 + P15 - P35 - P75);
	
	return feature;
}

}

