#ifndef IMAGE_WEIGHTED_SUM_MACHINE_INC
#define IMAGE_WEIGHTED_SUM_MACHINE_INC

#include "WeightedSumMachine.h"

namespace Torch {

/** Weighted-sum machine for images.

    @see WeightedSumMachine
    @author Sebastien Marcel (marcel@idiap.ch)
*/
class ImageWeightedSumMachine : public WeightedSumMachine
{
  public:
    // The machines inside the trainers
    Machine **machines;

    /** Constructor for training 
        @see WeightedSumMachine.h
    */
    ImageWeightedSumMachine(Trainer **trainers_, int n_trainers_, MeasurerList **trainers_measurers, real *weights_=NULL);

    /** Constructor for testing
        Machines are directly given (trainers not allocated)
    */ 
    ImageWeightedSumMachine(Machine **machines_, int n_machines_);

    //-----
    
    virtual void forward(Sequence *inputs);
    
    // assuming size of the output is 1
    virtual void fastforward(Sequence *inputs);

    virtual void reset();
    virtual void loadXFile(XFile *file);
    virtual void saveXFile(XFile *file);
    
    virtual ~ImageWeightedSumMachine();
};

}

#endif

