#ifndef REAL_STUMP_TRAINER_INC
#define REAL_STUMP_TRAINER_INC

#include "Trainer.h"
#include "RealStumpMachine.h"

namespace Torch {

/** This class is designed to train a RealStumpMachine (2-class problem) using histograms

    @author Yann Rodriguez (rodrig@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class RealStumpTrainer : public Trainer
{
private:
	/// number of bins for the LUT
	int n_bins;
	
	/// the LUT
	real *lut;

	// best feature
	int bestFeature;

	//
	real min_feat, max_feat;

	// best score
	real best_score;
	
	/// number of positive patterns
	int n_pos;

	/// number of negative patterns
	int n_neg;

	/// histogram of positive patterns
	int *histoPos;

	/// histogram of negative patterns
	int *histoNeg;

	// tmp arrays
	real *features;
	int *feature_index;
	real *targets;

	//------
	
	//
	void computeLogLikelihood(DataSet *data_, int feature_id);
	
public:
   	///
	bool verbose;

   	/// the real weak classifier
	RealStumpMachine *real_machine;
	
	///-----
	
	///
	RealStumpTrainer(RealStumpMachine *stump_machine_);

	///
	virtual void train(DataSet *data_, MeasurerList *measurers);

	///
	virtual ~RealStumpTrainer();
};

}

#endif
