#include "vitcTimeCode.h"

namespace Torch {

//#define GREYLEVEL	255
#define THRESHOLD	128
#define MAXIMUMLINE	35

vitcTimeCode::vitcTimeCode(int width_, int height_)
{
	width = width_;
	height = height_;
	
	for(int i = 0 ; i < 90 ; i++) VITCTimeCodeMask[i] = 0;
}

bool vitcTimeCode::reset(unsigned char *pixmap_)
{
	// TimeCode found with the existing VITCTimeCode
	if(ReadTimeCode(pixmap_)) return true;

	// VITCTimeCode wrong so reInit VITCTimeCode
  	InitVITCTimeCode(pixmap_);

	// Read the TimeCode with the new VITCTimeCode
  	return ReadTimeCode(pixmap_);
}

int vitcTimeCode::IsVITCTimeCode(unsigned char *VITCTimeCode_)
{
	int i, j;                      // Index varables       
	int SynHigh, SynLow;           // Check synchronization bit high, low
	int CRCRegister[8], CRC;       // CRC register and temp. var. for CRC check

	// Check for all first synchro bits == 1
	SynHigh=1;                     
	for(i=0; i<90; i+=10)
		SynHigh &= *(VITCTimeCode_+i);

	// Check for all second synchro bits == 0
	SynLow=0;
	for(i=1; i<90; i+=10)
		SynLow |= *(VITCTimeCode_+i);

	// Check CRC == 0  (g(x)=x8+1)
	for(i=0; i<8; i++)             // Initialisation
		CRCRegister[i]=0;

	for(i=0; i<90; i++)
	{
		CRC=CRCRegister[0];
		for(j=0; j<7; j++)     // Left shift
			CRCRegister[j]=CRCRegister[j+1];
		CRCRegister[7] = CRC^*(VITCTimeCode_+i);
	}

	CRC=0;                         // CRC result
	for(i=0; i<8; i++)
		CRC += CRCRegister[i];
	return (!CRC && SynHigh && !SynLow);
}

bool vitcTimeCode::InitVITCTimeCode(unsigned char *pixmap_)
{
	// Defines the mask applied to a picture to extract the 90 bits of the TimeCode
	// Definition done with the falling edge of the synchronization pulses

	unsigned long i=0, j=0;
	unsigned long FirstRaisingEdge, FirstFallingEdge;
	unsigned long PulseWidth, StartCenter, RealFallingEdge, EstimatedFallingEdge;

	while(i<height && i<MAXIMUMLINE)
	{													// For each line
    		// Distance between synchro pulses, rough approximation
		FirstRaisingEdge = i*width;												   	  	// Look first raising edge
		while((*(pixmap_+FirstRaisingEdge)<THRESHOLD) && (FirstRaisingEdge+1<(i+1)*width))
			FirstRaisingEdge++;	 						
														
		FirstFallingEdge = FirstRaisingEdge;						   							// Look first falling edge
		while((*(pixmap_+FirstFallingEdge)>THRESHOLD) && (FirstFallingEdge+1<(i+1)*width))
			FirstFallingEdge++;						
												
		PulseWidth = FirstFallingEdge - FirstRaisingEdge;										// Pulse width definition
		StartCenter = (FirstRaisingEdge + FirstFallingEdge-1)/2;				  			// Center pulse definition
		EstimatedFallingEdge = FirstFallingEdge +												// Est. of next synchro falling edge
		   ((width - (FirstRaisingEdge-i*width))/9);

		if(EstimatedFallingEdge<i*width || EstimatedFallingEdge >= (i+1)*width)
			EstimatedFallingEdge = (i+1)*width-1;											// If index not in the line, set to end of line

		// Distance between synchro pulses, fine approximation
		for(j=1; j<9; j++) 
		{
			RealFallingEdge = EstimatedFallingEdge - 2*PulseWidth + 1;						// Double rewind and double forward 
			if(RealFallingEdge<i*width || RealFallingEdge >= (i+1)*width)	// If index not in the line 
				RealFallingEdge = (i+1)*width-1;												// Then set to end of line

			while((*(pixmap_+RealFallingEdge-1)>THRESHOLD)<=(*(pixmap_+RealFallingEdge)>THRESHOLD)// Falling edge 
				&& (RealFallingEdge <= (EstimatedFallingEdge + 2*PulseWidth))	  		// Limit for synchro falling egde					
				&& (RealFallingEdge+1<(i+1)*width))										// Is next value to test in the line
				RealFallingEdge++;

			EstimatedFallingEdge = FirstFallingEdge + ((RealFallingEdge - FirstFallingEdge)/j)*(j+1);
			if(EstimatedFallingEdge<i*width || EstimatedFallingEdge >= (i+1)*width)	// If index not in the line 
				EstimatedFallingEdge = (i+1)*width-1;										// Then set to end of line
		}

		// Last synchro pulse found (RealFallingEdge): define VITCTimeCodeMask
		for(j=0; j<90; j++)
			VITCTimeCodeMask[j] = StartCenter + (unsigned long)((float)(j)*(float)((RealFallingEdge-FirstFallingEdge)/(float)80));

		if(ReadTimeCode(pixmap_))
			return true;													// TimeCode found, return 1 (init ok) 
		i++;
	}
	return false;														   // TimeCode not found, return 0 (init failed)
}

int vitcTimeCode::ReadTimeCode(unsigned char *pixmap_)
{
	int i;
	unsigned char VITCTimeCode[90];
	
	for(i=0; i<90; i++) 
	{											
	   	// Check VITCTimeCodeMask indexes (in pixmap)
		if(*(VITCTimeCodeMask+i)>= 0 && *(VITCTimeCodeMask+i)<width*height)
			*(VITCTimeCode+i) = *(pixmap_+*(VITCTimeCodeMask+i))>THRESHOLD;  
		else return 0;													// Impossible indexes, ReadTimeCode failed (0) 
	} 

	if(IsVITCTimeCode(VITCTimeCode)) 
	{
		// Define hour min sec and frame
		timecode[0] = 10*(*(VITCTimeCode+72)+2**(VITCTimeCode+73)) + 
				*(VITCTimeCode+62)+2**(VITCTimeCode+63)+4**(VITCTimeCode+64)+8**(VITCTimeCode+65);

		timecode[1] = 10*(*(VITCTimeCode+52)+2**(VITCTimeCode+53)+4**(VITCTimeCode+54)) + 
				*(VITCTimeCode+42)+2**(VITCTimeCode+43)+4**(VITCTimeCode+44)+8**(VITCTimeCode+45);
		if(timecode[1]>59)
			return 0;    

		timecode[2] = 10*(*(VITCTimeCode+32)+2**(VITCTimeCode+33)+4**(VITCTimeCode+34)) +
				*(VITCTimeCode+22)+2**(VITCTimeCode+23)+4**(VITCTimeCode+24)+8**(VITCTimeCode+25);
		if(timecode[2]>59)
			return 0; 

		timecode[3] = 10*(*(VITCTimeCode+12)+2**(VITCTimeCode+13)) +
				*(VITCTimeCode+2)+2**(VITCTimeCode+3)+4**(VITCTimeCode+4)+8**(VITCTimeCode+5); 
		if(timecode[3]>24)
			return 0;

		return 1;
	}

	return 0;
}

vitcTimeCode::~vitcTimeCode()
{
}

}
